# Generated by Django 3.0.4 on 2020-03-29 21:03

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_dramatiq_pg", "0001_initial"),
    ]

    operations = [
        migrations.RenameModel("QueuedJob", "BackgroundJob"),
    ]



from django.db.backends.signals import connection_created
from django.dispatch import receiver


@receiver(connection_created)
def add_dramatiq_schema(sender, connection, **kwargs):
    with connection.cursor() as cursor:
        cursor.execute("SELECT set_config('search_path', current_setting('search_path') || ', dramatiq', false);")



# Generated by Django 2.2.9 on 2020-02-16 23:57

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="QueuedJob",
            fields=[
                ("message_id", models.UUIDField(primary_key=True, serialize=False)),
                ("queue_name", models.TextField(default="default")),
                (
                    "state",
                    models.TextField(
                        choices=[
                            ('queued', 'Queued'),
                            ('consumed', 'Consumed'),
                            ('rejected', 'Rejected'),
                            ('done', 'Done'),
                        ],
                        default="queued",
                    ),
                ),
                ("mtime", models.DateTimeField()),
                ("message", django.contrib.postgres.fields.jsonb.JSONField()),
                ("result", django.contrib.postgres.fields.jsonb.JSONField()),
                ("result_ttl", models.DateTimeField()),
            ],
            options={"db_table": "queue", "managed": False},
        ),
        migrations.RunSQL(
            """
CREATE SCHEMA dramatiq;

CREATE TYPE dramatiq."state" AS ENUM (
  'queued',
  'consumed',
  'rejected',
  'done'
);

CREATE TABLE dramatiq.queue(
  message_id uuid PRIMARY KEY,
  queue_name TEXT NOT NULL DEFAULT 'default',
  "state" dramatiq."state",
  mtime TIMESTAMP WITH TIME ZONE DEFAULT (NOW() AT TIME ZONE 'UTC'),
  -- message as encoded by dramatiq.
  message JSONB,
  "result" JSONB,
  result_ttl  TIMESTAMP WITH TIME ZONE
) WITHOUT OIDS;

-- Index state and mtime together to speed up deletion. This can also speed up
-- statistics when VACUUM ANALYZE is recent enough.
CREATE INDEX ON dramatiq.queue ("state", mtime);
CREATE INDEX ON dramatiq.queue ((message->>'actor_name'));
        """
        ),
    ]



from django.contrib import admin

from .models import BackgroundJob


class ActorFilter(admin.SimpleListFilter):
    title = "Actor"
    parameter_name = "actor"
    lookup = "message__actor_name"

    def lookups(self, request, model_admin):
        qset = model_admin.model.objects.values_list(self.lookup, flat=True).distinct()
        return [(x, x) for x in qset]

    def queryset(self, request, queryset):
        value = request.GET.get(self.parameter_name)
        if value is None:
            return queryset
        return queryset.filter(**{self.lookup: value})


@admin.register(BackgroundJob)
class BackgroundJobAdmin(admin.ModelAdmin):
    list_display = (
        "message_id",
        "queue_name",
        "actor",
        "state",
    )
    list_filter = (
        "queue_name",
        ActorFilter,
        "state",
    )

    def actor(self, obj):
        return obj.message["actor_name"]

    actor.admin_order_field = "message__actor_name"



"""
Dramatiq queue worker

Pass this to the dramatiq command line :

$ dramatiq django_dramatiq_pg.worker
"""

import django
from django.conf import settings
from django.utils.module_loading import autodiscover_modules

django.setup()  # NOQA

autodiscover_modules(getattr(settings, "DRAMATIQ_ACTORS_MODULE", "actors"))



import dramatiq
from dramatiq_pg import PostgresBroker

from django.apps import AppConfig
from django.conf import settings
from django.utils.module_loading import import_string


class DramatiqConfig(AppConfig):
    name = "django_dramatiq_pg"
    verbose_name = "Dramatiq-PG"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.broker = None

    def ready(self):
        """
        Initialise our Broker when Django is ready.
        """
        from . import signals  # NOQA

        encoder = self.get_encoder()
        if encoder:
            dramatiq.set_encoder(encoder())

        options = self.get_broker_options()

        middleware = self.get_middleware()
        if middleware:
            options["middleware"] = middleware

        self.broker = PostgresBroker(**options)

        dramatiq.set_broker(self.broker)

        self.registry = self.get_registry()

        self.registry.bind_broker(self.broker)

    def get_encoder(self):
        encoder_path = getattr(settings, "DRAMATIQ_ENCODER", None)
        if encoder_path:
            return import_string(encoder_path)
        return None

    def get_broker_options(self):
        """This settings is _required_"""
        try:
            return settings.DRAMATIQ_BROKER["OPTIONS"]
        except KeyError:
            raise ValueError("No OPTIONS setting for DRAMATIQ_BROKER!")

    def get_middleware(self):
        try:
            middleware_config = settings.DRAMATIQ_BROKER["MIDDLEWARE"]
        except (AttributeError, KeyError):
            return None

        def resolve_class(name):
            if isinstance(name, str):
                return import_string(name)()
            return name

        middleware_list = [resolve_class(middleware) for middleware in middleware_config]

        return middleware_list

    def get_registry(self):
        """Import (or create) the Task Registry"""
        try:
            registry = import_string(settings.DRAMATIQ_REGISTRY)
        except AttributeError:
            from .registry import Registry

            registry = Registry()

        return registry



try:
    from dramatiq.registry import Registry

except ImportError:
    from dramatiq.actor import Actor, _queue_name_re

    class Registry:
        """
        A Registry allows defining a collection of Actors not directly bound
        to a Broker.

        This allows your code to declar Actors before configuring a Broker.
        """

        def __init__(self):
            self.actors = {}
            self.broker = None

        def actor(self, fn=None, *, actor_class=Actor, actor_name=None, queue_name="default", priority=0, **options):
            """
            Mimics `actor.actor` decorator, but skips the actor options check,
            and passes `self` as broker.
            """

            def decorator(fn):
                if not _queue_name_re.fullmatch(queue_name):
                    raise ValueError(
                        "Queue names must start with a letter or an underscore followed "
                        "by any number of letters, digits, dashes or underscores."
                    )

                return actor_class(
                    fn,
                    actor_name=actor_name or fn.__name__,
                    queue_name=queue_name,
                    priority=priority,
                    broker=self,
                    options=options,
                )

            if fn is None:
                return decorator
            return decorator(fn)

        def __getattr__(self, name):
            # Proxy everything else to our Broker, if set.
            return getattr(self.broker, name)

        def declare_actor(self, actor):
            """
            Intercept when Actor class tries to register itself.
            """
            if self.broker:
                self.broker.declare_actor(actor)
            else:
                self.actors[actor.actor_name] = actor

        def bind_broker(self, broker):
            self.broker = broker

            for actor_name, actor in self.actors.items():
                invalid_options = set(actor.options) - broker.actor_options
                if invalid_options:
                    invalid_options_list = ", ".join(invalid_options)
                    raise ValueError(
                        "Actor %s specified the following options which are "
                        "not supported by this broker: %s. Did you forget to "
                        " add a middleware to your Broker?" % (actor_name, invalid_options_list)
                    )

                broker.declare_actor(actor)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['django_dramatiq_pg', 'django_dramatiq_pg.migrations']

package_data = \
{'': ['*']}

install_requires = \
['django>=3.0', 'dramatiq-pg>=0.8.0']

setup_kwargs = {
    'name': 'django-dramatiq-pg',
    'version': '1.2.0',
    'description': 'Integration of Django with dramatiq-pg',
    'long_description': '==================\ndjango_dramatiq_pg\n==================\n\n.. rubric:: ``dramatiq-pg`` integration for django\n\nInstallation\n------------\n\n1. Install with pip\n\n   .. code-block:: sh\n\n    $ pip install django-dramatiq-pg\n\n2. Add to your ``INSTALLED_APPS`` list in settings.py\n\n   .. code-block:: python\n\n    INSTALLED_APPS = [\n        ...\n        \'django_dramatiq_pg\',\n    ]\n\n3. Create a Registry, and register your tasks\n\n   .. code-block:: python\n\n     from django_dramatiq_pg.registry import Registry\n\n     tasks = Registry()\n\n\n     @tasks.actor\n     def mytask():\n         ...\n\n4. Configure\n\n   .. code-block:: python\n\n    DRAMATIQ_BROKER = {\n        "OPTIONS": {\n            "url": "postgres:///mydb",\n        },\n        "MIDDLEWARE": [\n            "dramatiq.middleware.TimeLimit",\n            "dramatiq.middleware.Callbacks",\n            "dramatiq.middleware.Retries",\n        ],\n    }\n    DRAMATIC_REGISTRY = \'myapp.registry.tasks\'\n\n5. Start the worker process:\n\n   .. code-block:: sh\n\n    $ dramatiq django_dramatiq_pg.worker\n\nThis worker module will auto-discover any module called \'actors\' in\n``INSTALLED_APPS``.\n\nRegistry\n========\n\nIn a typical `dramatiq` application, the `Broker` is configured before any\ntasks are registered. However, as `Django` is in control of the intialisation\nsequence, there is an issue of ordering; the `actor` decorator assumes the\nbroker is already configured.\n\nTo resolve this, `django_dramatiq_pg` provides a `Registry` for your tasks,\nwhich is then bound to the `Broker` when Django initialises.\n\nIn your code, declare a `Registry` instance, and use its `.actor` method to\ndecorate your task functions. Then tell `django_dramatiq_pg` to use your\nregistry with the `DRAMATIQ_REGISTRY` setting.\n\nIf you do not specify one, `django_dramatiq_pg` will create one on start.\n\nThe registry can be accessed as the `.registry` attribute on the\n`django_dramatiq_pg` App instance.\n\nSettings\n--------\n\nDRAMATIQ_BROKER\n  A dict of options to pass when instantiating the broker.\n\nDRAMATIC_BROKER[\'OPTIONS\']\n  Arguments to pass to the Broker.\n\nDRAMATIC_BROKER[\'MIDDLEWARE\']\n  A list of middleware classes to be passed to the broker.\n\n  These can either be import strings, or instances.\n\nDRAMATIQ_ENCODER\n  Default: None\n\n  Import path for encoder class.\n\nDRAMATIQ_ACTORS_MODULE\n  Default: \'actors\'\n\n  Name of module use to auto-discover actors in INSTALLED_APPS.\n\nDRAMATIQ_REGISTRY\n\n  Import path for the task Registry instance.\n\n  This should refer to an instance of `django_dramatiq_pg.registry.Registry`.\n\n  This resolves the chicken/egg problem of declaring tasks before the broker is\n  configured.\n',
    'author': 'Curtis Maloney',
    'author_email': 'curtis@tinbrain.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/uptick/django-dramatiq-pg',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from django.contrib.postgres.fields import JSONField
from django.db import models
from django.utils import timezone


class BackgroundJob(models.Model):

    class STATE(models.TextChoices):
        QUEUED = "queued"
        CONSUMED = "consumed"
        REJECTED = "rejected"
        DONE = "done"

    message_id = models.UUIDField(primary_key=True)
    queue_name = models.TextField(default="default")
    state = models.CharField(max_length=16, default=STATE.QUEUED, choices=STATE.choices)
    mtime = models.DateTimeField(default=timezone.now)
    message = JSONField(blank=True, null=True)
    result = JSONField(blank=True, null=True)
    result_ttl = models.DateTimeField()

    class Meta:
        managed = False
        db_table = "queue"
        indexes = (models.Index(fields=["state", "mtime"]),)
